""" Requestors are components in charge of updating list of tags on a list of
    objects.
"""

from collections import defaultdict

from cloudcontrol.common.tql.db.tag import StaticTagInterface


def fetcher(*args):

    """ Decorator used to create a fetching method.
    """

    def decorator(func):
        func._fetcher_interfaces = args
        return func

    return decorator


class BaseRequestor(object):

    """ Base class for all requestors.
    """

    def __init__(self):
        self._fetcher_interface_mapping = {}
        for attr_name in dir(self):
            attr = getattr(self, attr_name)
            interfaces = getattr(attr, '_fetcher_interfaces', None)
            if interfaces is not None:
                for interface in interfaces:
                    if interface in self._fetcher_interface_mapping:
                        continue
                    self._fetcher_interface_mapping[interface] = attr

    def fetch(self, objects, tags):
        """ This method allow to fetch tags on objects.
        """
        map = defaultdict(lambda: [])
        for obj in objects:
            for tag_name in tags:
                try:
                    tag = obj[tag_name]
                except KeyError:
                    continue
                map[obj].append(tag)
        return self.fetch_map(map)

    def fetch_map(self, map):
        """ Fetch tags for specified objects.
        """
        # Dispatch each tag to a fetcher according to its type:
        fetcher_map = defaultdict(lambda: defaultdict(lambda: []))
        for obj, tags in map.iteritems():
            for tag in tags:
                # Search the matching fetcher:
                for interface, fetcher in self._fetcher_interface_mapping.iteritems():
                    if isinstance(tag, interface):
                        fetcher_map[fetcher][obj].append(tag)
                else:
                    pass  #XXX: fetcher not found
        # Call each fetcher:
        for fetcher, map in fetcher_map.iteritems():
            fetcher(map)


class StaticRequestor(BaseRequestor):

    """ A requestion which is able to request value for tags implementing the
        static interface.
    """

    @fetcher(StaticTagInterface)
    def fetch_static(self, map):
        """ Fetch tag using the StaticTagInterface.
        """
        for obj, tags in map.iteritems():
            for tag in tags:
                obj.set(tag.name, tag.value)
